import sqlite3
from collections import namedtuple
from pathlib import Path
import json
import random
import datetime

DATABASE =Path() / "data" / "xiuxian"

xiuxian_data = namedtuple("xiuxian_data", ["no", "user_id", "linggen", "level"])

UserDate = namedtuple("UserDate", ["id", "user_id", "stone", "root","root_type","level","power","create_time","is_sign","exp","user_name","level_up_cd"])
UserCd = namedtuple("UserCd", [ "user_id", "type", "create_time", "scheduled_time"])


def linggen_get():
    path_lg = DATABASE / "灵根.json"
    with open(path_lg,'r', encoding='utf-8') as e:
        a = e.read()
        data = json.loads(a)
        lg = random.choice(data)
        return lg['name'],lg['type']


class XiuxianDateManage:
    def __init__(self):
        self.database_path = DATABASE
        if not self.database_path.exists():
            self.database_path.mkdir(parents=True)
            self.database_path /= "xiuxian.db"
            self.conn = sqlite3.connect(self.database_path)
            # self._create_file()
        else:
            self.database_path /= "xiuxian.db"
            self.conn = sqlite3.connect(self.database_path)
        print(f"数据库已连接！")
        self._check_data()

    def close(self):
        self.conn.close()
        print("数据库关闭！")

    def _create_file(self) -> None:
        """创建数据库文件"""
        c = self.conn.cursor()
        c.execute('''CREATE TABLE User_xiuxian
                           (NO            INTEGER PRIMARY KEY UNIQUE,
                           USERID         TEXT     ,
                           level          INTEGER  ,
                           root           INTEGER
                           );''')
        c.execute('''''')
        c.execute('''''')
        self.conn.commit()

    def _check_data(self):
        """检查数据完整性"""
        c = self.conn.cursor()
        try:
            c.execute(f"select count(1) from user_cd")
        except:
            c.execute("""CREATE TABLE user_cd
                                    (user_id       INTEGER PRIMARY KEY UNIQUE,
                                    type           INTEGER,
                                    create_time    INTEGER,
                                    scheduled_time INTEGER);""")

        sql = "select * FROM level where name=?"
        c.execute(sql, ("元婴境圆满",))
        result = c.fetchone()
        if result:
            pass
        else:
            c.execute(f"DELETE from level")
            c.executescript(f"""INSERT INTO "main"."level" ("name", "power") VALUES ('江湖好手', 100);
INSERT INTO "main"."level" ("name", "power") VALUES ('练气境初期', 1000);
INSERT INTO "main"."level" ("name", "power") VALUES ('练气境中期', 2000);
INSERT INTO "main"."level" ("name", "power") VALUES ('练气境圆满', 3000);
INSERT INTO "main"."level" ("name", "power") VALUES ('筑基境初期', 8000);
INSERT INTO "main"."level" ("name", "power") VALUES ('筑基境中期', 9000);
INSERT INTO "main"."level" ("name", "power") VALUES ('筑基境圆满', 10000);
INSERT INTO "main"."level" ("name", "power") VALUES ('伪灵根', 0.8);
INSERT INTO "main"."level" ("name", "power") VALUES ('真灵根', 1);
INSERT INTO "main"."level" ("name", "power") VALUES ('天灵根', 1.2);
INSERT INTO "main"."level" ("name", "power") VALUES ('变异灵根', 1.2);
INSERT INTO "main"."level" ("name", "power") VALUES ('超灵根', 1.3);
INSERT INTO "main"."level" ("name", "power") VALUES ('混沌灵根', 1.4);
INSERT INTO "main"."level" ("name", "power") VALUES ('结丹境初期', 20000);
INSERT INTO "main"."level" ("name", "power") VALUES ('结丹境中期', 40000);
INSERT INTO "main"."level" ("name", "power") VALUES ('结丹境圆满', 80000);
INSERT INTO "main"."level" ("name", "power") VALUES ('元婴境初期', 160000);
INSERT INTO "main"."level" ("name", "power") VALUES ('元婴境中期', 360000);
INSERT INTO "main"."level" ("name", "power") VALUES ('元婴境圆满', 720000);""")

    def _get_id(self) -> int:
        """获取下一个id"""
        cur = self.conn.cursor()
        cur.execute('select id from user_xiuxian')
        result = cur.fetchall()
        print(result)
        return len(result) + 1

    @classmethod
    def close_dbs(cls):
        XiuxianDateManage().close()

    def _create_user(self, user_id: str,root:str,type:str,power:str,create_time,user_name) -> None:
        """在数据库中创建用户并初始化"""
        new_id = self._get_id()
        c = self.conn.cursor()
        sql = f"INSERT INTO user_xiuxian (id,user_id,stone,root,root_type,level,power,create_time,user_name) VALUES (?,?,0,?,?,'江湖好手',?,?,?)"
        c.execute(sql, (new_id, user_id,root,type,power,create_time,user_name))
        self.conn.commit()


    def get_user_message(self,user_id):
        '''获取用户信息'''
        cur = self.conn.cursor()
        sql = f"select * from user_xiuxian where user_id=?"
        cur.execute(sql, (user_id,))
        result = cur.fetchone()
        if not result:
            return None
        else:
            return UserDate(*result)

    def get_user_message2(self,user_id):
        '''获取用户信息'''
        cur = self.conn.cursor()
        sql = f"select * from user_xiuxian where user_name=?"
        cur.execute(sql, (user_id,))
        result = cur.fetchone()
        if not result:
            return None
        else:
            return UserDate(*result)

    def create_user(self,user_id,*args):
        cur = self.conn.cursor()
        sql = f"select * from user_xiuxian where user_id=?"
        cur.execute(sql, (user_id,))
        result = cur.fetchone()
        if not result:
            self._create_user(user_id,args[0],args[1],args[2],args[3],args[4])
            self.conn.commit()
            return '欢迎进入修仙世界的，你的灵根为：{},类型是：{},你的战力为：{},当前境界：江湖好手'.format(args[0], args[1], args[2],args[3])
        else:
            return '您已迈入修仙世界，输入我的修仙信息，获取数据吧！'

    def get_sign(self,user_id):
        '''获取用户签到信息'''
        cur = self.conn.cursor()
        sql = f"select is_sign from user_xiuxian where user_id=?"
        cur.execute(sql, (user_id,))
        result = cur.fetchone()
        if not result:
            return '修仙界没有你的足迹，输入 我要修仙 加入修仙世界吧！'
        elif result[0]==0:
            ls = random.randint(1,100)
            exp = random.randint(100,500)
            sql2 = f"UPDATE user_xiuxian SET is_sign=1,stone=stone+?,exp=exp+? WHERE user_id=?"
            cur.execute(sql2, (ls, exp,user_id))
            self.conn.commit()
            return '签到成功，获取{}块灵石,修为增加{}！'.format(ls,exp)
        elif result[0]==1:
            return '贪心的人是不会有好运的！'

    def ramaker(self,lg,type,user_id):
        """洗灵根"""
        cur = self.conn.cursor()

        # 查灵石
        sql_s = f"SELECT stone FROM user_xiuxian WHERE user_id=?"
        cur.execute(sql_s, (user_id,))
        result = cur.fetchone()
        if result[0]>=100:
            sql = f"UPDATE user_xiuxian SET root=?,root_type=?,stone=stone-100 WHERE user_id=?"
            cur.execute(sql, (lg, type,user_id))
            self.conn.commit()

            self.update_power(user_id)
            return "逆天之行，重获新生，新的灵根为：{}，类型为：{}".format(lg,type)
        else:
            return "你的灵石还不够呢，快去赚点灵石吧！"

    def get_type_power(self,name):
        """获取灵根或境界的战力和倍率"""
        cur = self.conn.cursor()
        sql = f"SELECT power FROM level WHERE name=?"
        cur.execute(sql, (name,))
        result = cur.fetchone()
        return result[0]

    def update_power(self,user_id) -> None:
        """更新战力"""
        UserMessage = self.get_user_message(user_id)
        cur = self.conn.cursor()
        sql = f"UPDATE user_xiuxian SET power=(SELECT power FROM level WHERE name=?)*(SELECT power FROM level WHERE name=?) WHERE user_id=?"
        cur.execute(sql, (UserMessage.root_type, UserMessage.level, user_id))
        self.conn.commit()

    def update_ls(self, user_id, price,key):
        """更新灵石  1为增加，2为减少"""
        cur = self.conn.cursor()

        if key == 1:
            sql = f"UPDATE user_xiuxian SET stone=stone+? WHERE user_id=?"
            cur.execute(sql, (price, user_id))
            self.conn.commit()
        elif key == 2:
            sql = f"UPDATE user_xiuxian SET stone=stone-? WHERE user_id=?"
            cur.execute(sql, (price, user_id))
            self.conn.commit()


    def get_ls_rank(self):
        """灵石排行榜"""
        sql = f"SELECT user_id,stone FROM user_xiuxian  WHERE stone>0 ORDER BY stone DESC LIMIT 5"
        cur = self.conn.cursor()
        cur.execute(sql,)
        result = cur.fetchall()
        return result


    def singh_remake(self):
        """重置签到"""
        sql = f"UPDATE user_xiuxian SET is_sign=0"
        cur = self.conn.cursor()
        cur.execute(sql, )
        self.conn.commit()

    def update_user_name(self,user_id,user_name):
        """更新用户道号"""
        cur = self.conn.cursor()
        get_name = f"select user_name from user_xiuxian where user_name=?"
        cur.execute(get_name, (user_name,))
        result = cur.fetchone()
        if result:
            return "已存在该道号！"
        else:
            sql = f"UPDATE user_xiuxian SET user_name=? where user_id=?"

            cur.execute(sql,(user_name, user_id))
            self.conn.commit()
            return '道友的道号更新成功拉~'

    def updata_level_cd(self,user_id):
        """更新破镜CD"""
        sql = f"UPDATE user_xiuxian SET level_up_cd=? where user_id=?"
        cur = self.conn.cursor()
        now_time = datetime.datetime.now()
        cur.execute(sql, (now_time, user_id))
        self.conn.commit()

    def updata_level(self,user_id,level_name):
        """更新境界"""
        sql = f"UPDATE user_xiuxian SET level=? where user_id=?"
        cur = self.conn.cursor()
        cur.execute(sql, (level_name, user_id))
        self.conn.commit()

    def get_user_cd(self, user_id):
        """
        获取用户操作CD
        :param user_id: QQ
        """
        sql = f"SELECT * FROM user_cd  WHERE user_id=?"
        cur = self.conn.cursor()
        cur.execute(sql, (user_id,))
        result = cur.fetchone()
        if result:
            return UserCd(*result)
        else:
            self.insert_user_cd(user_id,)
            return None

    def insert_user_cd(self, user_id) -> None:
        """
        添加用户至CD表
        :param user_id: qq
        :return:
        """
        sql = f"INSERT INTO user_cd (user_id) VALUES (?)"
        cur = self.conn.cursor()
        cur.execute(sql, (user_id,))
        self.conn.commit()

    def in_closing(self,user_id, the_type):
        """
        更新用户操作CD
        :param user_id: qq
        :param the_type: 0:无状态  1：闭关中  2：历练中
        :param the_time: 本次操作的时长
        :return:
        """
        if the_type == 1:
            now_time = datetime.datetime.now()
        elif the_type == 0:
            now_time = 0
        # scheduled_time = datetime.datetime.now() + datetime.timedelta(minutes=int(the_time))
        sql = f"UPDATE user_cd SET type=?,create_time=? where user_id=?"
        cur = self.conn.cursor()
        cur.execute(sql, (the_type, now_time, user_id))
        self.conn.commit()

    def out_closing(self,user_id, the_type):
        """出关状态更新"""
        pass

    def update_exp(self,user_id,exp):
        """增加修为"""
        sql = f"UPDATE user_xiuxian SET exp=exp+? where user_id=?"
        cur = self.conn.cursor()
        cur.execute(sql, (exp, user_id))
        self.conn.commit()

    def update_j_exp(self,user_id,exp):
        """减少修为"""
        sql = f"UPDATE user_xiuxian SET exp=exp-? where user_id=?"
        cur = self.conn.cursor()
        cur.execute(sql, (exp, user_id))
        self.conn.commit()


    def realm_top(self):
        """境界排行榜"""
        sql = f"""SELECT user_name,level,exp FROM user_xiuxian 
        WHERE user_name is NOT NULL
        ORDER BY CASE
        WHEN level = '元婴境圆满' THEN '10'
        WHEN level = '元婴境中期' THEN '11'
        WHEN level = '元婴境初期' THEN '12'
        WHEN level = '结丹境圆满' THEN '13'
        WHEN level = '结丹境中期' THEN '14'
        WHEN level = '结丹境初期' THEN '15'
        WHEN level = '筑基境圆满' THEN '16'
        WHEN level = '筑基境中期' THEN '17'
        WHEN level = '筑基境初期' THEN '18'
        WHEN level = '练气境圆满' THEN '19'
        WHEN level = '练气境中期' THEN '20'
        WHEN level = '练气境初期' THEN '21'
        WHEN level = '江湖好手' THEN '22'
        ELSE level END ASC,exp DESC LIMIT 5"""
        cur = self.conn.cursor()
        cur.execute(sql, )
        result = cur.fetchall()
        mess = f"✨位面境界排行榜TOP5✨\n"
        num = 0
        for i in result:
            num += 1
            mess += f"第{num}位 {i[0]} {i[1]},修为{i[2]}\n"

        return mess

    def stone_top(self):
        sql = f"SELECT user_name,stone FROM user_xiuxian WHERE user_name is NOT NULL ORDER BY stone DESC LIMIT 5"
        cur = self.conn.cursor()
        cur.execute(sql, )
        result = cur.fetchall()
        mess = f"✨位面灵石排行榜TOP5✨\n"
        num = 0
        for i in result:
            num += 1
            mess += f"第{num}位  {i[0]}  灵石：{i[1]}枚\n"

        return mess


class XiuxianJsonDate:
    def __init__(self):
        self.root_jsonpath = DATABASE / "灵根.json"
        self.level_jsonpath = DATABASE / "突破概率.json"


    def linggen_get(self):
        with open(self.root_jsonpath, 'r', encoding='utf-8') as e:
            a = e.read()
            data = json.loads(a)
            lg = random.choice(data)
            return lg['name'], lg['type']

    def level_rate(self, level):
        with open(self.level_jsonpath, 'r', encoding='utf-8') as e:
            a = e.read()
            data = json.loads(a)
            return data[0][level]


class OtherSet:

    def __init__(self):
        self.level = ['江湖好手', '练气境初期','练气境中期', '练气境圆满', '筑基境初期', '筑基境中期', '筑基境圆满',
                      '结丹境初期', '结丹境中期', '结丹境圆满', '元婴境初期', '元婴境中期', '元婴境圆满']


    def set_closing_type(self,user_level):
        list_all = len(self.level) - 1
        now_index = self.level.index(user_level)
        if list_all == now_index:
            return "道友已是最高境界，无法突破！"
        is_updata_level = self.level[now_index + 1]
        need_exp = XiuxianDateManage().get_type_power(is_updata_level)
        return need_exp

    def get_type(self,user_exp,rate,user_level):
        list_all = len(self.level) - 1
        now_index = self.level.index(user_level)
        if list_all == now_index:
            return "道友已是最高境界，无法突破！"

        is_updata_level = self.level[now_index + 1]
        need_exp = XiuxianDateManage().get_type_power(is_updata_level)

        #判断修为是否足够突破
        if user_exp >= need_exp:
            pass
        else:
            return "道友的修为不足以突破！距离下次突破需要{}修为！突破境界为：{}".format(need_exp-user_exp,is_updata_level)

        success_rate = True if random.randint(0, 100) < rate else False

        if success_rate:

            return [self.level[now_index + 1]]
        else:
            return '失败'

if __name__ == '__main__':
    # paths = r"D:\yuzi_bot\yuzi_bot\data\xiuxian\突破概率.json"
    # with open(paths, 'r', encoding='utf-8') as e:
    #     a = e.read()
    #     data = json.loads(a)
    #     print(data[0]['练气境初期'])

    apath = r"G:\yuzi_bot\yuzi_bot\data\xiuxian\xiuxian.db"
    conn = sqlite3.connect(apath)
    sql = f"""SELECT user_name,level,exp FROM user_xiuxian 
WHERE user_name is NOT NULL
ORDER BY CASE
WHEN level = '筑基境圆满' THEN '1'
WHEN level = '筑基境中期' THEN '2'
WHEN level = '筑基境初期' THEN '3'
WHEN level = '练气境圆满' THEN '4'
WHEN level = '练气境中期' THEN '5'
WHEN level = '练气境初期' THEN '6'
WHEN level = '江湖好手' THEN '7'
ELSE level END ASC,exp DESC LIMIT 5"""
    cur = conn.cursor()
    cur.execute(sql,)
    result = cur.fetchall()
    mess = f"位面境界排行榜TOP5\n"
    num=0
    for i in result:
        num+=1
        mess += F"TOP{num}:{i[0]},境界：{i[1]},修为：{i[2]}\n"

    print(mess)



import re
from itertools import chain
from nonebot.log import logger
from nonebot import get_driver
from nonebot import on_command,require,on_message
from nonebot.params import CommandArg, RawCommand,Depends, Arg, ArgStr, RegexMatched
from nonebot.adapters.onebot.v11 import Bot, Event, GROUP, GROUP_ADMIN, GROUP_OWNER, Message, MessageEvent, GroupMessageEvent,MessageSegment
from .xiuxian2_handle import XiuxianDateManage,linggen_get,XiuxianJsonDate,OtherSet
from datetime import datetime
import random
from .xiuxian_opertion import gamebingo

scheduler = require("nonebot_plugin_apscheduler").scheduler


__xiuxian_version__ = "v0.0.1"
__xiuxian_notes__ = f'''
修仙模拟器帮助信息:
指令：
1、我要修仙：进入修仙模式
2、我的修仙信息：获取修仙数据
3、修仙签到：获取灵石及修为
4、重入仙途：重置灵根数据，每次100灵石
5、#金银阁：猜大小，赌灵石
6、改名xx：修改你的道号
7、突破：修为足够后，可突破境界（一定几率失败）
8、闭关、出关：修炼增加修为，挂机功能
9、送灵石+数量+道号或者艾特对应人
10、排行榜：修仙排行榜，灵石排行榜
10、其他功能to do中 
'''.strip()

driver = get_driver()


run_xiuxian = on_command('我要修仙',priority=5)
xiuxian_message = on_command('我的修仙信息',aliases={'我的存档'},priority=5)
rename = on_command('改名',priority=5)
restart = on_command('再入仙途',aliases={'重新修仙'},priority=5)
package = on_command('我的纳戒',aliases={'升级纳戒'},priority=5)
sign_in = on_command('修仙签到',priority=5)
dufang = on_command('#金银阁',aliases={'金银阁'},priority=5)
dice = on_command('大',aliases={'小'},priority=5)
price = on_command('押注',priority=5)
help_in = on_command('修仙帮助',priority=5)
remaker = on_command('重入仙途',priority=5)
use = on_command('#使用',priority=5)
buy = on_command('#购买',priority=5)
level_rank = on_command('境界排行',aliases={'修仙排行榜','排行榜'},priority=5)
stone_rank = on_command('灵石排行',aliases={'灵石排行榜'},priority=5)
time_mes = on_message(priority=999)
remaname = on_command('改名',priority=5)
level_up = on_command('突破',priority=5)
in_closing = on_command('闭关',priority=5)
out_closing = on_command('出关',priority=5)
give_stone = on_command('送灵石', priority=5)

race = {}
sql_message = XiuxianDateManage()

@run_xiuxian.handle()
async def _(bot: Bot, event: GroupMessageEvent,args: Message = CommandArg()):
    user_id = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())
    # text = args.extract_plain_text().strip()   获取命令后面的信息
    user_name = event.sender.card if event.sender.card else event.sender.nickname
    name, type = linggen_get()
    rate  = sql_message.get_type_power(type)
    power = 100 * float(rate)
    create_time = str(datetime.now())

    msg = sql_message.create_user(user_id,name,type,int(power),create_time,user_name)
    await run_xiuxian.finish(msg,at_sender=True)

@xiuxian_message.handle()
async def _(event: GroupMessageEvent):
    user_id = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())


    mess = sql_message.get_user_message(user_id)
    user_name = mess.user_name
    if user_name:
        pass
    else:
        user_name = '无名氏(发送改名+道号更新)'
    if mess:
        msg = f'''{user_name}道友的信息
灵根为：{mess[3]}
灵根类型为：{mess[4]}
当前境界：{mess[5]}
当前灵石：{mess[2]}
当前修为：{mess.exp}
你的战力为：{mess[6]}'''
    else:
        msg = '未曾踏入修仙世界，输入 我要修仙 加入我们，看破这世间虚妄!'

    await run_xiuxian.finish(msg,at_sender=True)

@sign_in.handle()
async def _(event: GroupMessageEvent):
    user_id = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())
    result = sql_message.get_sign(user_id)
    await sign_in.send(result, at_sender=True)


@help_in.handle()
async def _(event: GroupMessageEvent):
    msg = __xiuxian_notes__
    await help_in.send(msg, at_sender=True)

@dice.handle()
async def _(bot: Bot, event: GroupMessageEvent, args: Message = CommandArg(), cmd: Message = RawCommand()):
    global race
    message = event.message
    user_id = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())
    try:
        race[group_id]
    except KeyError:
        await price.finish()

    if race[group_id].player[0] == user_id:
        pass
    else:
        await dice.finish('吃瓜道友请不要捣乱！！！')

    price_num = race[group_id].price
    if price_num==0:
        await dice.finish('道友押注失败,请发送【押注+数字】押注！', at_sender=True)


    if str(message) == '大' or str(message) == '小':
        pass
    else:
        await dice.finish('请输入正确的结果【大】或者【小】！')

    value = random.randint(1, 6)
    msg = Message("[CQ:dice,value={}]".format(value))

    if value >= 4:
        if str(message)=='大':
            del race[group_id]
            sql_message.update_ls(user_id,price_num,1)
            await dice.send(msg)
            await dice.finish('最终结果为{}，你猜对了，收获灵石{}块'.format(value,price_num),at_sender=True)
        else:
            del race[group_id]
            sql_message.update_ls(user_id, price_num, 2)
            await dice.send(msg)
            await dice.finish('最终结果为{}，你猜错了，损失灵石{}块'.format(value, price_num),at_sender=True)
    elif value <= 3:
        if str(message) == '大':
            del race[group_id]
            sql_message.update_ls(user_id, price_num, 2)
            await dice.send(msg)
            await dice.finish('最终结果为{}，你猜错了，损失灵石{}块'.format(value, price_num),at_sender=True)
        else:
            del race[group_id]
            sql_message.update_ls(user_id, price_num, 1)
            await dice.send(msg)
            await dice.finish('最终结果为{}，你猜对了，收获灵石{}块'.format(value, price_num),at_sender=True)


@dufang.handle()
async def _(bot: Bot, event: GroupMessageEvent, args: Message=CommandArg(), cmd: Message=RawCommand()):
    global race
    user_id = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())
    user_message = sql_message.get_user_message(user_id)
    if user_message:
        if user_message.stone==0:
            await price.finish(f"走开走开，没钱还来玩！", at_sender=True)
    else:
        await price.finish(f"本阁没有这位道友的信息！输入【我要修仙】加入吧！", at_sender=True)

    try:
        if race[group_id].start == 1 and race[group_id].player[0] == user_id:
            await dufang.finish(f"道友的活动已经开始了，发送【押注+数字】参与")
        elif race[group_id].start == 1 and race[group_id].player[0] != user_id:
            await dufang.finish(f"已有其他道友进行中")
    except KeyError:
        pass
    race[group_id] = gamebingo()
    race[group_id].start_change(1)
    race[group_id].add_player(user_id)
    race[group_id].time = datetime.now()
    await dufang.finish(f'发送【押注+数字】参与',at_sender=True)


@price.handle()
async def _(bot: Bot, event: GroupMessageEvent, args: Message=CommandArg(), cmd: Message=CommandArg()):
    global race
    user_id = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())
    msg = args.extract_plain_text().strip()

    user_message = sql_message.get_user_message(user_id)
    try:
        race[group_id]
    except KeyError:
        await price.finish(f"金银阁未开始，请输入【金银阁】开场",at_sender=True)
    try:
        if race[group_id].player[0] == user_id:
            pass
        else:
            await price.finish('吃瓜道友请不要捣乱！')
    except KeyError:
        await price.finish()
    if msg:
        price_num = msg
        if race[group_id].price != 0:
            await price.finish('钱财离手，不可退回！', at_sender=True)
        elif int(user_message.stone) < int(price_num):
            await price.finish('道友的金额不足，请重新输入！')
        elif price_num.isdigit():
            race[group_id].add_price(int(price_num))
        else:
            await price.finish('请输入正确的金额！')
    else:
        await price.finish(f"请输入押注金额", at_sender=True)

    out_msg = f'押注完成，发送【大】或者【小】 参与本局游戏！'
    await price.finish(out_msg, at_sender=True)



@remaker.handle()
async def _(bot: Bot, event: GroupMessageEvent, args: Message=CommandArg(), cmd: Message=RawCommand()):
    user_id = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())

    name, type = linggen_get()
    result = sql_message.ramaker(name,type,user_id)
    await remaker.send(message=result,at_sender=True)


@level_rank.handle()
async def _():
    rank = sql_message.realm_top()
    await level_rank.send(message=rank)

@stone_rank.handle()
async def _():
    rank = sql_message.stone_top()
    await stone_rank.send(message=rank)


# 重置每日签到
@scheduler.scheduled_job(
    "cron",
    hour=0,
    minute=0,
)
async def _():
    sql_message.singh_remake()
    logger.info("每日修仙签到重置成功！")


@time_mes.handle()
async def _(bot: Bot,event: GroupMessageEvent):
    global race
    group_id = await get_group_id(event.get_session_id())
    # print(event.sender.card if event.sender.card else event.sender.nickname)
    try:
        if race[group_id]:
            race_time = race[group_id].time
            time_now = datetime.now()
            if (time_now - race_time).seconds >30:
                del race[group_id]
                await time_mes.finish('太久没押注开始，被挤走了')
            else:
                pass
        else:
            pass
    except KeyError:
        pass

@remaname.handle()
async def _(bot: Bot,event: GroupMessageEvent,args: Message = CommandArg()):
    user_id = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())
    user_name = args.extract_plain_text().strip()
    if sql_message.get_user_message(user_id) is None:
        await in_closing.finish("修仙界没有道友的信息，请输入【我要修仙】加入！")

    mes = sql_message.update_user_name(user_id,user_name)
    await remaname.finish(mes)


@in_closing.handle()
async def _(bot: Bot, event: GroupMessageEvent,args: Message = CommandArg()):
    user_type = 1
    user_id = event.get_user_id()

    if sql_message.get_user_message(user_id) is None:
        await in_closing.finish("修仙界没有道友的信息，请输入【我要修仙】加入！")
    user_cd_message = sql_message.get_user_cd(user_id)

    if user_cd_message is None:
        sql_message.in_closing(user_id, user_type)
        await in_closing.finish("进入闭关状态，如需出关，发送【出关】！", at_sender=True)
    elif user_cd_message.type == 0:
        sql_message.in_closing(user_id, user_type)
        await in_closing.finish("进入闭关状态，如需出关，发送【出关】！", at_sender=True)
    elif user_cd_message.type == 1:
        await in_closing.finish("已经在闭关中，请输入【出关】结束！", at_sender=True)
    elif user_cd_message.type == 2:
        await in_closing.finish("已经在历练中，请输入【结束历练】结束！", at_sender=True)

@out_closing.handle()
async def _(bot: Bot, event: GroupMessageEvent,args: Message = CommandArg()):
    user_type = 0
    user_id = event.get_user_id()

    user_mes = sql_message.get_user_message(user_id)
    level = user_mes.level
    use_exp = user_mes.exp

    max_exp = int(OtherSet().set_closing_type(level))
    user_get_exp_max = max_exp - use_exp
    now_time = datetime.now()
    user_cd_message = sql_message.get_user_cd(user_id)

    if user_cd_message is None:
        await out_closing.finish("没有查到道友的信息，修炼发送【闭关】，进入修炼状态！",at_sender=True)
    elif user_cd_message.type == 0:
        await out_closing.finish("道友现在什么都没干呢~", at_sender=True)
    elif user_cd_message.type == 1:
        in_closing_time = datetime.strptime(user_cd_message.create_time, "%Y-%m-%d %H:%M:%S.%f")
        exp_time = (now_time - in_closing_time).seconds // 60
        exp = exp_time * 10
        if exp >= user_get_exp_max:
            sql_message.in_closing(user_id,user_type)
            sql_message.update_exp(user_id,user_get_exp_max)
            await out_closing.finish("闭关结束，本次闭关到达上限，共增加修为：{}".format(user_get_exp_max), at_sender=True)
        else:
            sql_message.in_closing(user_id, user_type)
            sql_message.update_exp(user_id, exp)
            await out_closing.finish("闭关结束，共闭关{}分钟，本次闭关增加修为：{}".format(exp_time, exp),at_sender=True)
    elif user_cd_message.type == 2:
        await out_closing.finish("已经在历练中，请输入【结束历练】结束！")



async def get_group_id(session_id):
    res = re.findall("_(.*)_", session_id)
    group_id = res[0]
    return group_id

@level_up.handle()
async def update_level(event: GroupMessageEvent):
    user_id = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())

    user_mes = sql_message.get_user_message(user_id)

    level_cd = user_mes.level_up_cd
    if level_cd:
        time_now = datetime.now()
        cd = (time_now - datetime.strptime(level_cd, '%Y-%m-%d %H:%M:%S.%f')).seconds
        if cd < 6000:
            await level_up.finish('目前无法突破，还需要{}分钟'.format(100 - (cd//60)))
    else:
        pass

    level_name = user_mes.level  #境界
    exp = user_mes.exp   #修为

    level_rate = XiuxianJsonDate().level_rate(level_name)  #对应境界突破的概率

    le = OtherSet().get_type(exp, level_rate, level_name)

    if le=='失败':
        sql_message.updata_level_cd(user_id)
        percentage = random.randint(1, 10)
        now_exp = int(int(exp) * (percentage / 100))
        sql_message.update_j_exp(user_id, now_exp)
        await level_up.finish('道友突破失败,境界受损,修为减少{}，过段时间再突破吧！'.format(now_exp))
    elif type(le)==list:
        sql_message.updata_level(user_id,le[0])
        sql_message.update_power(user_id)
        sql_message.updata_level_cd(user_id)
        await level_up.finish('恭喜道友突破{}成功'.format(le[0]))
    else:
        await level_up.finish(le)


@give_stone.handle()
async def _(bot: Bot, event: GroupMessageEvent,args: Message = CommandArg()):
    user_id = event.get_user_id()
    user_message = sql_message.get_user_message(user_id)
    if user_message is None:
        await give_stone.finish('修仙界没有你的信息！请输入我要修仙，踏入修行')

    user_stone_num = user_message.stone
    give_qq = None  #艾特的时候存到这里
    give_name = None
    msg = args.extract_plain_text().strip()

    stone_num = re.findall("\d+", msg)  ##灵石数
    nick_name = re.findall("\D+", msg)  ##道号

    if stone_num:
        pass
    else:
        await give_stone.finish('请输入正确的灵石数量！')

    give_stone_num = stone_num[0]
    print(give_stone_num)
    print(user_stone_num)
    if int(give_stone_num) > int(user_stone_num):
        await give_stone.finish('道友的灵石不够，请重新输入！')

    for arg in args:
        if arg.type == "at":
            give_qq = arg.data.get('qq','')

    if give_qq:
        if give_qq == user_id :
            await give_stone.finish("请不要送灵石给自己！")
        else:
            give_user = sql_message.get_user_message(give_qq)
            if give_user:
                sql_message.update_ls(user_id, give_stone_num, 2)  # 减少用户灵石
                give_stone_num2 = int(give_stone_num) * 0.03
                num = int(give_stone_num) - int(give_stone_num2)
                sql_message.update_ls(give_qq, num, 1)   # 增加用户灵石

                await give_stone.finish(
                    "共赠送{}枚灵石给{}道友！收取手续费{}枚".format(give_stone_num,give_qq, int(give_stone_num2)))
            else:
                await give_stone.finish("对方未踏入修仙界，不可赠送！")

    if nick_name:
        give_message = sql_message.get_user_message2(nick_name[0])
        if give_message:
            if give_message.user_name == user_message.user_name:
                await give_stone.finish("请不要送灵石给自己！")
            else:
                sql_message.update_ls(user_id, give_stone_num, 2)  # 减少用户灵石
                give_stone_num2 = int(give_stone_num) * 0.03
                num = int(give_stone_num) - int(give_stone_num2)
                sql_message.update_ls(give_message.user_id, num, 1)  # 增加用户灵石
                await give_stone.finish(
                    "共赠送{}枚灵石给{}道友！收取手续费{}枚".format(give_stone_num, give_message.user_name, int(give_stone_num2)))
        else:
            await give_stone.finish("对方未踏入修仙界，不可赠送！")

    else:
        await give_stone.finish("未获取道号信息，请输入正确的道号！")















from distutils.core import setup
import setuptools
setuptools.setup(
  name = 'nonebot_plugin_xiuxian',         # How you named your package folder (MyLib)
  packages = ['nonebot_plugin_xiuxian'],   # Chose the same as "name"
  version = '0.2.7',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'xiuxian模拟器',   # Give a short description about your library
  author = 'PinkCat',                   # Type in your name
  author_email = '578043031@qq.com',      # Type in your E-Mail
  url = 'https://github.com/s52047qwas/nonebot_plugin_xiuxian',   # Provide either the link to your github or to your website
  # packages=setuptools.find_packages(),
  keywords = ['plugin', 'nonebot2', 'xiuxian'],   # Keywords that define your package best
  install_requires=[
          'nonebot2',
          'nonebot-adapter-onebot'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3.9',
  ],
)

